﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable.Problems {
  [Item("Scriptable Problem (single-objective)", "Represents a single-objective problem that can be scripted.")]
  [Creatable("1 Test")]
  [StorableClass]
  public sealed class SingleObjectiveScriptableProblem : SingleObjectiveProgrammableProblem<IEncoding> {

    private FixedValueParameter<SingleObjectiveProblemDefinitionScript> SingleObjectiveProblemScriptParameter {
      get { return (FixedValueParameter<SingleObjectiveProblemDefinitionScript>)Parameters["ProblemScript"]; }
    }

    public SingleObjectiveProblemDefinitionScript ProblemScript {
      get { return SingleObjectiveProblemScriptParameter.Value; }
    }

    public ISingleObjectiveProblemDefinition ProblemDefinition {
      get { return SingleObjectiveProblemScriptParameter.Value; }
    }

    private SingleObjectiveScriptableProblem(SingleObjectiveScriptableProblem original, Cloner cloner)
      : base(original, cloner) {
      RegisterEvents();
      ProblemScript.Compile();
    }
    public override IDeepCloneable Clone(Cloner cloner) { return new SingleObjectiveScriptableProblem(this, cloner); }


    [StorableConstructor]
    private SingleObjectiveScriptableProblem(bool deserializing) : base(deserializing) { }

    public SingleObjectiveScriptableProblem()
      : base() {
      Parameters.Add(new FixedValueParameter<SingleObjectiveProblemDefinitionScript>("ProblemScript", "Defines the problem.", new SingleObjectiveProblemDefinitionScript() { Name = Name }));
      RegisterEvents();


    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterEvents();
      ProblemScript.Compile();
    }

    private void RegisterEvents() {
      ProblemScript.ProblemDefinitionChanged += (o, e) => OnProblemDefinitionChanged();
    }

    private void OnProblemDefinitionChanged() {
      Encoding = ProblemDefinition.Encoding;
    }

    public override bool Maximization {
      get { return ProblemDefinition.Maximization; }
    }

    public override double Evaluate(Individual individual, IRandom random) {
      return ProblemDefinition.Evaluate(individual, random);
    }

    public override void Analyze(Individual[] individuals, double[] qualities, ResultCollection results) {
      ProblemDefinition.Analyze(individuals, qualities, results);
    }
    public override IEnumerable<Individual> GetNeighbors(Individual individual, IRandom random) {
      return ProblemDefinition.GetNeighbors(individual, random);
    }
  }
}
